<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model\Total\Creditmemo;

use Magento\Sales\Model\Order\Creditmemo;
use Magento\Tax\Model\Config;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class Cashondelivery extends AbstractTotal
{
    
    /**
     * Tax config
     *
     * @var \Magento\Tax\Model\Config
     */
    protected $taxConfig;
    
    /**
     * @var PriceCurrencyInterface
     */
    protected $priceCurrency;
    
    public function __construct(
        Config $taxConfig,
        PriceCurrencyInterface $priceCurrency,
        array $data = []
    ) {
        parent::__construct($data);
        $this->taxConfig = $taxConfig;
        $this->priceCurrency = $priceCurrency;
    }
    
    
    public function collect(Creditmemo $creditmemo)
    {
        $order = $creditmemo->getOrder();
        
        // amounts without tax
        $orderMspCodAmount = $order->getMspCodAmount() - $order->getMspCodTaxAmount();
        $orderBaseMspCodAmount = $order->getBaseMspCodAmount() - $order->getBaseMspCodTaxAmount();
        $allowedAmount = $orderMspCodAmount;
        $baseAllowedAmount = $orderBaseMspCodAmount;

        // amounts including tax
        $orderMspCodInclTax = $order->getMspCodAmount();
        $orderBaseMspCodInclTax = $order->getBaseMspCodAmount();
        $allowedTaxAmount = $order->getMspCodTaxAmount();
        $baseAllowedTaxAmount = $order->getBaseMspCodTaxAmount();
        $allowedAmountInclTax = $allowedAmount + $allowedTaxAmount;
        $baseAllowedAmountInclTax = $baseAllowedAmount + $baseAllowedTaxAmount;

        // for the credit memo
        $mspCodAmount = $baseMspCodAmount = $mspCodInclTax = $baseMspCodInclTax = 0;
        
        // For the conditional logic, we will either use amounts that always include tax -OR- never include tax.
        // The logic uses the 'base' currency to be consistent with what the user (admin) provided as input.
        $useAmountsWithTax = $this->isSuppliedCodAmountInclTax($order);
        

        
        // Check if the desired shipping amount to refund was specified (from invoice or another source or from creation form).
        if ($creditmemo->hasBaseMspCodAmount()) {
            
            // Since the user (admin) supplied 'desiredAmount' it already has tax -OR- does not include tax
            $desiredAmount = $this->priceCurrency->round($creditmemo->getBaseMspCodAmount());
            $maxAllowedAmount = ($useAmountsWithTax ? $baseAllowedAmountInclTax : $baseAllowedAmount);
            $originalTotalAmount = ($useAmountsWithTax ? $orderBaseMspCodInclTax : $orderBaseMspCodAmount);
            
            // Note: ($x < $y + 0.0001) means ($x <= $y) for floats
            if ($desiredAmount < $this->priceCurrency->round($maxAllowedAmount) + 0.0001) {
                // since the admin is returning less than the allowed amount, compute the ratio being returned
                $ratio = 0;
                if ($originalTotalAmount > 0) {
                    $ratio = $desiredAmount / $originalTotalAmount;
                }
                // capture amounts without tax
                // Note: ($x > $y - 0.0001) means ($x >= $y) for floats
                if ($desiredAmount > $maxAllowedAmount - 0.0001) {
                    $mspCodAmount = $allowedAmount;
                    $baseMspCodAmount = $baseAllowedAmount;
                } else {
                    $mspCodAmount = $this->priceCurrency->round($orderMspCodAmount * $ratio);
                    $baseMspCodAmount = $this->priceCurrency->round($orderBaseMspCodAmount * $ratio);
                }
                $mspCodInclTax = $this->priceCurrency->round($orderMspCodInclTax * $ratio);
                $baseMspCodInclTax = $this->priceCurrency->round($orderBaseMspCodInclTax * $ratio);
            } else {
                $maxAllowedAmount = $order->getBaseCurrency()->format($maxAllowedAmount, null, false);
                throw new \Magento\Framework\Exception\LocalizedException(
                    __('Maximum cash on delivery amount allowed to refund is: %1', $maxAllowedAmount)
                );
            }
            
        } else {
            $mspCodAmount = $allowedAmount;
            $baseMspCodAmount = $baseAllowedAmount;
            $mspCodInclTax = $this->priceCurrency->round($allowedAmountInclTax);
            $baseMspCodInclTax = $this->priceCurrency->round($baseAllowedAmountInclTax);
        }
        
        if ($useAmountsWithTax) {
            $creditmemo->setMspCodAmount($mspCodInclTax);
            $creditmemo->setBaseMspCodAmount($baseMspCodInclTax);
        } else {
            $creditmemo->setMspCodAmount($mspCodAmount);
            $creditmemo->setBaseMspCodAmount($baseMspCodAmount);
        }
        
        if ($this->_canApplyTotal($order)) {
            $creditmemo->setGrandTotal($creditmemo->getGrandTotal() + $creditmemo->getMspCodAmount());
            $creditmemo->setBaseGrandTotal($creditmemo->getBaseGrandTotal() + $creditmemo->getBaseMspCodAmount());
        }

        return $this;
    }
    
    
    
    /**
     * Returns whether the user specified a shipping amount that already includes tax
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    private function isSuppliedCodAmountInclTax($order)
    {
        // returns true if we are only displaying shipping including tax, otherwise returns false
        return $this->taxConfig->displayCartPricesInclTax($order->getStoreId());
    }
}
